@@ -26,6 +26,8 @@ import string
 import debconf
 from datetime import datetime
 
+import apt_pkg
+
 # avoid hanging forever (LP: #1243090)
 import socket
 socket.setdefaulttimeout(60)
@@ -176,6 +178,18 @@ def download_file(uri, sha256_hashsum):
     """
     download_dir = os.path.join(STAMPDIR, "partial")
     dest_file = os.path.join(download_dir, os.path.basename(uri))
+    try:
+        with open(dest_file, 'rb') as dest_file_obj:
+            # apt_pkg can directly hash from file obj, let's use that instead
+            # of hashlib
+            real_sha256 = apt_pkg.sha256sum(dest_file_obj)
+            if real_sha256 == sha256_hashsum:
+                return dest_file
+            else:
+                os.remove(dest_file)
+    except FileNotFoundError:
+        pass
+
     ret = subprocess.call(
         ["/usr/lib/apt/apt-helper",
          "download-file", uri, dest_file, "SHA256:" + sha256_hashsum])
